@@ -1,6 +1,6 @@
 // BreakControl.cc
 //
-// Copyright (C) 2001, 2002, 2003, 2004 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 // GNU General Public License for more details.
 //
 
-static const char rcsid[] = "$Id: BreakControl.cc,v 1.4 2004/04/15 22:09:31 rcaelers Exp $";
+static const char rcsid[] = "$Id: BreakControl.cc,v 1.10 2006/01/26 21:30:23 rcaelers Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -66,11 +66,10 @@ BreakControl::BreakControl(BreakId id, Core *c, AppInterface *app, Timer *timer)
   max_number_of_postpones(2),
   ignorable_break(true),
   config_ignorable_break(true),
-  insist_policy(BreakInterface::INSIST_POLICY_HALT),
-  active_insist_policy(BreakInterface::INSIST_POLICY_INVALID),
   fake_break(false),
   fake_break_count(0),
-  user_abort(false)
+  user_abort(false),
+  delayed_abort(false)
 {
   set_ignorable_break(ignorable_break);
 
@@ -105,9 +104,8 @@ BreakControl::heartbeat()
     }
   else
     {
-      // Unless the timer has it's own activity monitor.
-      ActivityMonitorInterface *monitor = core->get_activity_monitor();
-      ActivityState activity_state = monitor->get_current_state();
+      // Unless the timer has its own activity monitor.
+      ActivityState activity_state = core->get_current_monitor_state();
       is_idle = (activity_state != ACTIVITY_ACTIVE);
     }
 
@@ -122,7 +120,12 @@ BreakControl::heartbeat()
 
     case STAGE_DELAYED:
       {
-        if (is_idle)
+        if (delayed_abort)
+          {
+            // User become active during delayed break.
+            goto_stage(STAGE_SNOOZED);
+          }
+        else if (is_idle)
           {
             // User is idle.
             goto_stage(STAGE_TAKING);
@@ -133,6 +136,10 @@ BreakControl::heartbeat()
     case STAGE_PRELUDE:
       {
         assert(application != NULL);
+
+        TRACE_MSG("prelude time = " << prelude_time);
+
+
         update_prelude_window();
         application->refresh_break_window();
 
@@ -153,19 +160,10 @@ BreakControl::heartbeat()
                 // Final prelude, force break.
                 goto_stage(STAGE_TAKING);
               }
-            else
+            else 
               {
-                // Snooze break.
-                break_stage = STAGE_PRELUDE; // FIXME: hack to avoid race. will fix later.
-                bool delayed = application->delayed_hide_break_window();
-                if (!delayed)
-                  {
-                    goto_stage(STAGE_SNOOZED);
-                  }
-                else
-                  {
-                    goto_stage(STAGE_DELAYED);
-                  }
+                // Delay break.
+                goto_stage(STAGE_DELAYED);
               }
           }
         else if (prelude_time == 20)
@@ -210,13 +208,17 @@ BreakControl::goto_stage(BreakStage stage)
   switch (stage)
     {
     case STAGE_DELAYED:
+      {
+        ActivityMonitorInterface *monitor = core->get_activity_monitor();
+        monitor->set_listener(this);
+      }
       break;
       
     case STAGE_NONE:
       {
         // Teminate the break.
         application->hide_break_window();
-        defrost();
+        core->defrost();
 
         if (break_stage == STAGE_TAKING)
           {
@@ -257,7 +259,7 @@ BreakControl::goto_stage(BreakStage stage)
           {
             post_event(CORE_EVENT_SOUND_BREAK_IGNORED);
           }
-        defrost();
+        core->defrost();
       }
       break;
 
@@ -280,8 +282,7 @@ BreakControl::goto_stage(BreakStage stage)
         application->hide_break_window();
 
         // "Innocent until proven guilty".
-        ActivityMonitorInterface *monitor = core->get_activity_monitor();
-        monitor->force_idle();
+        core->force_idle();
         break_timer->stop_timer();
 
 	// Check if we have reached the maximum number of preludes and force
@@ -320,10 +321,7 @@ BreakControl::goto_stage(BreakStage stage)
             post_event(event);
           }
 
-        //RC: testing... if (insist_break)
-          {
-            freeze();
-          }
+        core->freeze();
       }
       break;
     }
@@ -383,6 +381,7 @@ BreakControl::start_break()
   user_initiated = false;
   prelude_time = 0;
   user_abort = false;
+  delayed_abort = false;
   
   reached_max_prelude = max_number_of_preludes >= 0 && prelude_count + 1 >= max_number_of_preludes;
   reached_max_postpone = max_number_of_postpones >= 0 && postponable_count + 1 >= max_number_of_postpones;
@@ -398,8 +397,7 @@ BreakControl::start_break()
       // Starting break with prelude.
       
       // Idle until proven guilty.
-      ActivityMonitorInterface *monitor = core->get_activity_monitor();
-      monitor->force_idle();
+      core->force_idle();
       break_timer->stop_timer();
 
       // Update statistics.
@@ -421,14 +419,15 @@ BreakControl::start_break()
 
 //! Starts the break without preludes.
 void
-BreakControl::force_start_break()
+BreakControl::force_start_break(bool initiated_by_user)
 {
   TRACE_ENTER_MSG("BreakControl::force_start_break", break_id);
 
   fake_break = false;
-  user_initiated = true;
+  user_initiated = initiated_by_user;
   prelude_time = 0;
   user_abort = false;
+  delayed_abort = false;
   
   if (break_timer->is_auto_reset_enabled())
     {
@@ -442,6 +441,11 @@ BreakControl::force_start_break()
           fake_break_count = break_timer->get_auto_reset();
         }
     }
+
+  if (!break_timer->get_activity_sensitive())
+    {
+      break_timer->force_idle();
+    }
   
   goto_stage(STAGE_TAKING);
 
@@ -483,7 +487,7 @@ BreakControl::suspend_break()
   TRACE_ENTER_MSG("BreakControl::suspend_break", break_id);
 
   goto_stage(STAGE_NONE);
-  defrost();
+  core->defrost();
   
   TRACE_EXIT();
 }
@@ -548,16 +552,6 @@ BreakControl::postpone_break()
 }
 
 
-void
-BreakControl::stop_prelude()
-{
-  if (break_stage == STAGE_DELAYED)
-    {
-      goto_stage(STAGE_SNOOZED);
-    }
-}
-
-
 //! Skips the active break.
 /*!
  *  Skipping a break resets the break timer.
@@ -568,13 +562,14 @@ BreakControl::skip_break()
   // This is to avoid a skip sound...
   user_abort = true;
 
-  // Reset the restbreak timer.
   if (break_id == BREAK_ID_DAILY_LIMIT)
     {
+      // Make sure the daily limit remains silent after skipping.
       break_timer->inhibit_snooze();
     }
   else
     {
+      // Reset the restbreak timer.
       break_timer->reset_timer();
     }
   
@@ -586,6 +581,11 @@ BreakControl::skip_break()
   stop_break(false);
 }
 
+void
+BreakControl::stop_prelude()
+{
+  delayed_abort = true;
+}
 
 //! Sets the maximum number of preludes. 
 /*!
@@ -606,7 +606,6 @@ BreakControl::set_max_postpone(int m)
 }
 
 
-
 //! Sets the ignorable-break flags
 /*!
  *  A break that is ignorable has a skip/postpone button.
@@ -618,40 +617,6 @@ BreakControl::set_ignorable_break(bool i)
 }
 
 
-//! Sets the insist policy.
-/*!
- *  The insist policy determines what to do when the user is active while
- *  taking a break.
- */
-void
-BreakControl::set_insist_policy(BreakInterface::InsistPolicy p)
-{
-  TRACE_ENTER_MSG("BreakControl::set_insist_policy", p);
-
-  if (active_insist_policy != BreakInterface::INSIST_POLICY_INVALID &&
-      insist_policy != p)
-    {
-      TRACE_MSG("refreeze " << active_insist_policy);
-      defrost();
-      insist_policy = p;
-      freeze();
-    }
-  else
-    {
-      insist_policy = p;
-    }
-  TRACE_EXIT();
-}
-
-
-//! Gets the insist policy.
-BreakInterface::InsistPolicy
-BreakControl::get_insist_policy() const
-{
-  return insist_policy;
-}
-
-
 //! Creates and shows the break window.
 void
 BreakControl::break_window_start()
@@ -690,82 +655,15 @@ BreakControl::prelude_window_start()
 }
 
 
-//! Excecutes the insist policy.
-void
-BreakControl::freeze()
+
+
+bool
+BreakControl::action_notify()
 {
-  TRACE_ENTER_MSG("BreakControl::freeze", insist_policy);
-  BreakInterface::InsistPolicy policy = insist_policy;
-
-  // OBSOLETE (i hope)
-//   if (!insist_break)
-//     {
-//       // This break does not user input. When the user moves the mouse,
-//       // we timer should start running... this is a hack...
-//       if (policy == BreakInterface::INSIST_POLICY_HALT)
-//         {
-//           policy = BreakInterface::INSIST_POLICY_RESET;
-//         }
-//     }
-  
-  
-  switch (policy)
-    {
-    case BreakInterface::INSIST_POLICY_IGNORE:
-      {
-        // Ignore all activity during break by suspending the activity monitor.
-        ActivityMonitorInterface *monitor = core->get_activity_monitor();
-        monitor->suspend();
-      }
-      break;
-    case BreakInterface::INSIST_POLICY_HALT:
-      {
-        // Halt timer when the user is active.
-        core->set_freeze_all_breaks(true);
-      }
-      break;
-    case BreakInterface::INSIST_POLICY_RESET:
-      // reset the timer when the user becomes active.
-      // default.
-      break;
-      
-    default:
-      break;
-    }
-
-  active_insist_policy = policy;
-  TRACE_EXIT();
-}
-
-
-//! Undoes the insist policy.
-void
-BreakControl::defrost()
-{
-  TRACE_ENTER_MSG("BreakControl::defrost", active_insist_policy);
-  
-  switch (active_insist_policy)
-    {
-    case BreakInterface::INSIST_POLICY_IGNORE:
-      {
-        // Resumes the activity monitor.
-        ActivityMonitorInterface *monitor = core->get_activity_monitor();
-        monitor->resume();
-      }
-      break;
-    case BreakInterface::INSIST_POLICY_HALT:
-      {
-        // Desfrost timers.
-        core->set_freeze_all_breaks(false);
-      }
-      break;
-      
-    default:
-      break;
-    }
-
-  active_insist_policy = BreakInterface::INSIST_POLICY_INVALID;
+  TRACE_ENTER("GUI::action_notify");
+  core->stop_prelude(break_id);
   TRACE_EXIT();
+  return false;   // false: kill listener.
 }
 
 
@@ -816,8 +714,7 @@ BreakControl::set_state_data(bool active, const BreakStateData &data)
           TRACE_MSG("Break active");
 
           // Idle until proven guilty.
-          ActivityMonitorInterface *monitor = core->get_activity_monitor();
-          monitor->force_idle();
+          core->force_idle();
           break_timer->stop_timer();
       
           goto_stage(STAGE_TAKING);
@@ -840,8 +737,7 @@ BreakControl::set_state_data(bool active, const BreakStateData &data)
               TRACE_MSG("PRELUDE");
 
               // Idle until proven guilty.
-              ActivityMonitorInterface *monitor = core->get_activity_monitor();
-              monitor->force_idle();
+              core->force_idle();
               break_timer->stop_timer();
   
               goto_stage(STAGE_PRELUDE);
