@@ -1,6 +1,6 @@
 // Display.cc
 //
-// Copyright (C) 2002, 2003, 2004 Rob Caelers & Raymond Penners
+// Copyright (C) 2002, 2003, 2004, 2005 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -13,19 +13,22 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: System.cc,v 1.3 2004/04/03 20:16:16 rcaelers Exp $
+// $Id: System.cc,v 1.9 2006/01/22 16:44:28 rcaelers Exp $
 //
 
 #include "config.h"
 #include <glib.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 
 #include "System.hh"
 #include "debug.hh"
 
 #if defined(HAVE_X)
 #include <X11/X.h>
+#include <X11/Xproto.h>
+#include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #endif
 
@@ -117,6 +120,7 @@ System::is_lockable()
 void
 System::lock()
 {
+  TRACE_ENTER("System::lock");
   if (is_lockable())
     {
 #if defined(HAVE_X)
@@ -128,6 +132,7 @@ System::lock()
       (*lock_func)();
 #endif  
     }
+  TRACE_EXIT();
 }
 
 bool
@@ -198,7 +203,7 @@ System::init(
     }
   else if ((lock = g_find_program_in_path("xlock")))
     {
-      xlock = g_strdup_printf("%s --display \"%s\"",
+      xlock = g_strdup_printf("%s -display \"%s\"",
                               lock, display);
     }
   g_free(lock);
@@ -269,7 +274,6 @@ has_wm_state (Display *display, Window xwindow)
 static bool
 look_for_kdesktop_recursive (Display *display, Window xwindow)
 {
-  
   Window ignored1, ignored2;
   Window *children;
   unsigned int n_children;
@@ -288,12 +292,11 @@ look_for_kdesktop_recursive (Display *display, Window xwindow)
       
     XGetClassHint (display, xwindow, &ch);
       
-      
     if (ch.res_name)
       XFree (ch.res_name);
       
     if (ch.res_class) {
-      if (strcmp (ch.res_class, "kdesktop") == 0) {
+      if (strcasecmp (ch.res_class, "kdesktop") == 0) {
         XFree (ch.res_class);
         return true;
       }
