@@ -1,6 +1,6 @@
 // TimerBoxControl.cc --- Timers Widgets
 //
-// Copyright (C) 2001, 2002, 2003, 2004 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 // GNU General Public License for more details.
 //
 
-static const char rcsid[] = "$Id: TimerBoxControl.cc,v 1.3 2004/02/05 22:12:57 dotsphinx Exp $";
+static const char rcsid[] = "$Id: TimerBoxControl.cc,v 1.11 2005/04/30 22:42:14 rcaelers Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -51,9 +51,11 @@ const std::string TimerBoxControl::CFG_KEY_TIMERBOX_IMMINENT = "/imminent";
 
 //! Constructor.
 TimerBoxControl::TimerBoxControl(std::string n, TimerBoxView &v) :
+  view(&v),
   cycle_time(10),
   name(n),
-  view(&v)
+  force_duration(0),
+  force_empty(false)
 {
   init();
 }
@@ -72,20 +74,41 @@ TimerBoxControl::~TimerBoxControl()
 void
 TimerBoxControl::update()
 {
+  CoreInterface *core = CoreFactory::get_core();
+  OperationMode mode = core->get_operation_mode();
+
   if (reconfigure)
     {
       // Configuration was changed. reinit.
       init_table();
+
+      operation_mode = mode;
+      init_icon();
+
       reconfigure = false;
     }
   else
     {
-      time_t t = time(NULL);
-      if (t % cycle_time == 0)
+      if (force_duration == 0)
         {
-          init_table();
-          cycle_slots();
+          time_t t = time(NULL);
+          if (t % cycle_time == 0)
+            {
+              init_table();
+              cycle_slots();
+            }
         }
+      else
+        {
+          force_duration--;
+        }
+    }
+
+  // Update visual feedback of operating mode.
+  if (mode != operation_mode)
+    {
+      operation_mode = mode;
+      init_icon();
     }
   
   // Update the timer widgets.
@@ -94,19 +117,35 @@ TimerBoxControl::update()
 }
 
 
+void
+TimerBoxControl::force_cycle()
+{
+  force_duration = cycle_time;
+  init_table();
+  cycle_slots();
+}
+
+void
+TimerBoxControl::set_force_empty(bool s)
+{
+  force_empty = s;
+}
+
+
 //! Initializes the timerbox.
 void
 TimerBoxControl::init()
 {
   TRACE_ENTER("TimerBoxControl::init");
 
+  CoreInterface *core = CoreFactory::get_core();
+  
   // Listen for configugration changes.
   ConfiguratorInterface *config = CoreFactory::get_configurator();
   config->add_listener(TimerBoxControl::CFG_KEY_TIMERBOX + name, this);
 
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
-      CoreInterface *core = CoreFactory::get_core();
       BreakInterface *break_data = core->get_break(BreakId(i));
       config->add_listener("gui/breaks/"
                            + break_data->get_name()
@@ -151,6 +190,10 @@ TimerBoxControl::update_widgets()
       num_peers = dist_manager->get_number_of_peers();
     }
 #endif
+
+  //FIXME: duplicate
+  char *labels[] = { _("Micro-break"), _("Rest break"), _("Daily limit") };
+  string tip = "Workrave";
   
   for (int count = 0; count < BREAK_ID_SIZEOF; count++)
     {
@@ -163,7 +206,8 @@ TimerBoxControl::update_widgets()
       int primary_val, primary_max;
       TimeBarInterface::ColorId secondary_color;
       int secondary_val, secondary_max;
-      
+
+#ifdef LETS_SEE_HOW_WORKRAVE_BEHAVES_WITHOUT_THIS      
       if (!node_master && num_peers > 0)
         {
           text = _("Inactive");
@@ -175,6 +219,7 @@ TimerBoxControl::update_widgets()
           secondary_max = 60;
         }
       else
+#endif
         {
           if (timer == NULL)
             {
@@ -189,7 +234,7 @@ TimerBoxControl::update_widgets()
           time_t breakDuration = timer->get_auto_reset();
           time_t idleTime = timer->get_elapsed_idle_time();
           bool overdue = (maxActiveTime < activeTime);
-          
+
           // Set the text
           if (timer->is_limit_enabled() && maxActiveTime != 0)
             {
@@ -200,6 +245,10 @@ TimerBoxControl::update_widgets()
               text = Text::time_to_string(activeTime);
             }
 
+          tip += "\n";
+          tip += labels[count];
+          tip += ": " + text;
+          
           // And set the bar.
           secondary_val = secondary_max = 0;
           secondary_color = TimeBarInterface::COLOR_ID_INACTIVE;
@@ -234,6 +283,30 @@ TimerBoxControl::update_widgets()
                          primary_color, primary_val, primary_max,
                          secondary_color, secondary_val, secondary_max);
     }
+  view->set_tip(tip);
+}
+
+
+void
+TimerBoxControl::init_icon()
+{
+  switch (operation_mode)
+    {
+    case OPERATION_MODE_NORMAL:
+      view->set_icon(TimerBoxView::ICON_NORMAL);
+      break;
+      
+    case OPERATION_MODE_SUSPENDED:
+      view->set_icon(TimerBoxView::ICON_SUSPENDED);
+      break;
+      
+    case OPERATION_MODE_QUIET:
+      view->set_icon(TimerBoxView::ICON_QUIET);
+      break;
+
+    default:
+      break;
+    }
 }
 
 
@@ -243,29 +316,38 @@ TimerBoxControl::init_table()
 {
   TRACE_ENTER("TimerBoxControl::init_table");
 
-  // Determine what breaks to show.
-  for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+  if (force_empty)
     {
-      init_slot(i);
-    }
-
-  // New content.
-  int slot = 0;
-  for (int i = 0; i < BREAK_ID_SIZEOF; i++)
-    {
-      int cycle = break_slot_cycle[i];
-      int id = break_slots[i][cycle]; // break id
-      if (id != -1)
+      for (int i = 0; i < BREAK_ID_SIZEOF; i++)
         {
-          view->set_slot(BreakId(id), slot);
-          slot++;
+          view->set_slot(BREAK_ID_NONE, i);
         }
     }
-  for (int i = slot; i < BREAK_ID_SIZEOF; i++)
+  else
     {
-      view->set_slot(BREAK_ID_NONE, i);
+      // Determine what breaks to show.
+      for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+        {
+          init_slot(i);
+        }
+  
+      // New content.
+      int slot = 0;
+      for (int i = 0; i < BREAK_ID_SIZEOF; i++)
+        {
+          int cycle = break_slot_cycle[i];
+          int id = break_slots[i][cycle]; // break id
+          if (id != -1)
+            {
+              view->set_slot(BreakId(id), slot);
+              slot++;
+            }
+        }
+      for (int i = slot; i < BREAK_ID_SIZEOF; i++)
+        {
+          view->set_slot(BREAK_ID_NONE, i);
+        }
     }
-
   TRACE_EXIT();
 }
 
@@ -277,21 +359,17 @@ TimerBoxControl::init_slot(int slot)
   // TRACE_ENTER_MSG("TimerBoxControl::init_slot", slot);
   int count = 0;
   int breaks_id[BREAK_ID_SIZEOF];
-  bool stop = false;
 
   // Collect all timers for this slot.
-  for (int i = 0; !stop && i < BREAK_ID_SIZEOF; i++)
+  for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
-      // TRACE_MSG("1 " << i);
       CoreInterface *core = CoreFactory::get_core();
       BreakInterface *break_data = core->get_break(BreakId(i));
 
       bool on = break_data->get_break_enabled();
-      // TRACE_MSG("2 " << on);
       
       if (on && break_position[i] == slot && !(break_flags[i] & BREAK_HIDE))
         {
-          // TRACE_MSG("3");
           breaks_id[count] = i;
           break_flags[i] &= ~BREAK_SKIP;
           count++;
@@ -302,34 +380,27 @@ TimerBoxControl::init_slot(int slot)
   time_t first = 0;
   int first_id = -1;
     
-  // TRACE_MSG("4");
   for (int i = 0; i < count; i++)
     {
-      // TRACE_MSG("5");
       int id = breaks_id[i];
       int flags = break_flags[id];
 
-      // TRACE_MSG("6 " << id << " " << flags);
-      
       CoreInterface *core = CoreFactory::get_core();
       BreakInterface *break_data = core->get_break(BreakId(id));
       TimerInterface *timer = break_data->get_timer();
 
-      // TRACE_MSG("7");
-      
       time_t time_left = timer->get_limit() - timer->get_elapsed_time();
         
       // Exclude break if not imminent.
-      if (flags & BREAK_WHEN_IMMINENT && time_left > break_imminent_time[id])
+      if (flags & BREAK_WHEN_IMMINENT && time_left > break_imminent_time[id] &&
+          force_duration == 0)
         {
-          // TRACE_MSG("8");
           break_flags[id] |= BREAK_SKIP;
         }
 
       // update first imminent timer.
       if (!(flags & BREAK_SKIP) && (first_id == -1 || time_left < first))
         {
-          // TRACE_MSG("9");
           first_id = id;
           first = time_left;
         }
@@ -339,16 +410,13 @@ TimerBoxControl::init_slot(int slot)
   // Exclude break if not first.
   for (int i = 0; i < count; i++)
     {
-      // TRACE_MSG("10 " << i);
       int id = breaks_id[i];
       int flags = break_flags[id];
 
       if (!(flags & BREAK_SKIP))
         {
-          // TRACE_MSG("11");
-          if (flags & BREAK_WHEN_FIRST && first_id != id)
+          if (flags & BREAK_WHEN_FIRST && first_id != id && force_duration == 0)
             {
-              // TRACE_MSG("12");
               break_flags[id] |= BREAK_SKIP;
             }
         }
@@ -360,43 +428,34 @@ TimerBoxControl::init_slot(int slot)
   int breaks_left = 0;
   for (int i = 0; i < count; i++)
     {
-      // TRACE_MSG("13 " << i);
-
       int id = breaks_id[i];
       int flags = break_flags[id];
 
       if (!(flags & BREAK_SKIP))
         {
-          // TRACE_MSG("14");
-          if (flags & BREAK_EXCLUSIVE && have_one)
+          if (flags & BREAK_EXCLUSIVE && have_one && force_duration == 0)
             {
-              // TRACE_MSG("15");
               break_flags[id] |= BREAK_SKIP;
             }
 
           have_one = true;
         }
       
-      // TRACE_MSG("15");
       if (!(flags & BREAK_SKIP))
         {
-          // TRACE_MSG("16");
           breaks_left++;
         }
     }
 
   if (breaks_left == 0)
     {
-      // TRACE_MSG("17");
       for (int i = 0; i < count; i++)
         {
-          // TRACE_MSG("18");
           int id = breaks_id[i];
           int flags = break_flags[id];
           
           if (flags & BREAK_DEFAULT && flags & BREAK_SKIP)
             {
-              // TRACE_MSG("19");
               break_flags[id] &= ~BREAK_SKIP;
               breaks_left = 1;
               break;
@@ -404,13 +463,11 @@ TimerBoxControl::init_slot(int slot)
         }
     }
 
-  // TRACE_MSG("20");
   for (int i = 0; i < BREAK_ID_SIZEOF; i++)
     {
       break_slots[slot][i] = -1;
     }
 
-  // TRACE_MSG("21");
   int new_count = 0;
   for (int i = 0; i < count; i++)
     {
@@ -423,8 +480,6 @@ TimerBoxControl::init_slot(int slot)
           new_count++;
         }
     }
-  // TRACE_EXIT();
-
 }
 
 
