@@ -13,7 +13,7 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: Frame.cc,v 1.2 2004/04/03 15:55:33 rcaelers Exp $
+// $Id: Frame.cc,v 1.3 2004/10/31 19:21:45 rcaelers Exp $
 //
 
 #ifdef HAVE_CONFIG_H
@@ -114,8 +114,10 @@ Frame::set_frame_flashing(int delay)
 bool
 Frame::on_timer()
 {
+  TRACE_ENTER("Frame::on_timer");
   set_frame_visible(! frame_visible);
   flash_signal_src(frame_visible);
+  TRACE_EXIT();
   return true;
 }
 
