@@ -1,6 +1,6 @@
 // GUI.cc --- The WorkRave GUI
 //
-// Copyright (C) 2001, 2002, 2003, 2004 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 // GNU General Public License for more details.
 //
 
-static const char rcsid[] = "$Id: GUI.cc,v 1.11 2004/05/13 16:57:21 rcaelers Exp $";
+static const char rcsid[] = "$Id: GUI.cc,v 1.21 2005/04/25 18:56:22 rcaelers Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -61,8 +61,15 @@ static const char rcsid[] = "$Id: GUI.cc,v 1.11 2004/05/13 16:57:21 rcaelers Exp
 
 #ifdef HAVE_GNOME
 #include "RemoteControl.hh"
+#ifdef HAVE_GNOMEMM
 #include "libgnomeuimm/wrap_init.h"
 #endif
+#endif
+
+#ifdef HAVE_KDE
+#include <dcopclient.h>
+#include <kapp.h>
+#endif
 
 #ifdef WIN32
 #include "crashlog.h"
@@ -70,6 +77,7 @@ static const char rcsid[] = "$Id: GUI.cc,v 1.11 2004/05/13 16:57:21 rcaelers Exp
 #endif
 
 #include <gtkmm/main.h>
+#include <gtkmm/messagedialog.h>
 
 #include <glibmm/refptr.h>
 
@@ -133,12 +141,6 @@ GUI::~GUI()
 
   ungrab();
   
-  if (dispatcher != NULL)
-    {
-      dispatch_connection.disconnect();
-      delete dispatcher;
-    }
-
   if (core != NULL)
     {
       // FIXME: cannot delete interface. delete core;
@@ -161,9 +163,10 @@ GUI::~GUI()
 void
 GUI::restbreak_now()
 {
-  core->force_break(BREAK_ID_REST_BREAK);
+  core->force_break(BREAK_ID_REST_BREAK, true);
 }
 
+
 //! The main entry point.
 void
 GUI::main()
@@ -175,14 +178,15 @@ GUI::main()
   __try1(exception_handler);
 #endif
 
-//    char *x = NULL;
-//    *x = 1;
   Gtk::Main kit(argc, argv);
   
 #ifdef HAVE_GNOME
   init_gnome();
 #endif
-
+#ifdef HAVE_KDE
+  init_kde();
+#endif
+    
 #ifdef WIN32
   // Win32 needs this....
   if (!g_thread_supported())
@@ -321,6 +325,7 @@ GUI::init_gnome()
   
   gnome_init("workrave", VERSION, argc, argv);
 
+#ifdef HAVE_GNOMEMM
   Gnome::UI::wrap_init();
 
   Gnome::UI::Client *client = Gnome::UI::Client::master_client();
@@ -331,8 +336,10 @@ GUI::init_gnome()
     }
   
   TRACE_EXIT();
+#endif  
 }
 
+#ifdef HAVE_GNOMEMM
 void
 GUI::on_die()
 {
@@ -394,9 +401,24 @@ GUI::on_save_yourself(int phase, Gnome::UI::SaveStyle save_style, bool shutdown,
   return true;  
 }
 
-#endif  
+#endif // defined HAVE_GNOMEMM
+#endif // defined HAVE_GNOME
 
 
+#ifdef HAVE_KDE
+void
+GUI::init_kde()
+{
+  // FIXME: memory leaks in here.
+  TRACE_ENTER("GUI::init_kde");
+  KApplication *a = new KApplication(argc, argv, "Workrave");
+  bool rc = kapp->dcopClient()->attach();
+  TRACE_MSG(rc);
+
+  TRACE_EXIT();
+}
+#endif
+
 //! Initializes messages hooks.
 void
 GUI::init_debug()
@@ -568,7 +590,7 @@ GUI::init_gtk_multihead()
   int num_screens = display->get_n_screens();
 
   TRACE_MSG("screens = " << num_screens);
-  if (num_screens > 1)
+  if (num_screens >= 1)
     {
       for (int i = 0; i < num_screens; i++)
         {
@@ -576,7 +598,7 @@ GUI::init_gtk_multihead()
 #ifdef HAVE_GTKMM24
           if (screen)
 #else
-            if (!screen.is_null())
+          if (!screen.is_null())
 #endif        
               {
                 new_num_heads += screen->get_n_monitors();
@@ -592,7 +614,7 @@ GUI::init_gtk_multihead()
 #ifdef HAVE_GTKMM24
           if (screen)
 #else
-            if (!screen.is_null())
+          if (!screen.is_null())
 #endif        
               {
                 int num_monitors = screen->get_n_monitors();
@@ -681,6 +703,9 @@ GUI::init_gui()
   // Setup the window hints module.
   WindowHints::init();
 
+  tooltips = manage(new Gtk::Tooltips());
+  tooltips->enable();
+
   // The main status window.
   main_window = new MainWindow();
   
@@ -689,14 +714,8 @@ GUI::init_gui()
   applet_window = new AppletWindow();
 #endif
   
-  tooltips = manage(new Gtk::Tooltips());
-  tooltips->enable();
-
   // Periodic timer.
   Glib::signal_timeout().connect(MEMBER_SLOT(*this, &GUI::on_timer), 1000);
-
-  dispatcher = new Dispatcher;
-  dispatch_connection = dispatcher->connect(SigC::slot_class(*this, &GUI::on_activity));
 }
 
 
@@ -710,10 +729,19 @@ GUI::init_remote_control()
       g_error (_("I could not initialize Bonobo"));
     }
 
-  RemoteControl::get_instance();
+  RemoteControl *control = RemoteControl::get_instance();
+  if (control == NULL)
+    {
+      Gtk::MessageDialog dialog("Could not initialize Workrave. Is Workrave already running?",
+                                false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+      dialog.show();
+      dialog.run();
+      exit(1);
+    }
   
   BonoboGenericFactory *factory;
-  factory = bonobo_generic_factory_new("OAFIID:GNOME_Workrave_Factory", workrave_component_factory, NULL);
+  factory = bonobo_generic_factory_new("OAFIID:GNOME_Workrave_Factory",
+                                       workrave_component_factory, NULL);
   bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
 #endif
 }
@@ -727,8 +755,7 @@ GUI::create_break_window(HeadInfo &head, BreakId break_id, bool ignorable)
   BlockMode block_mode = get_block_mode();
   if (break_id == BREAK_ID_MICRO_BREAK)
     {
-      ret = new MicroBreakWindow(head, core->get_timer(BREAK_ID_REST_BREAK),
-                                 ignorable, block_mode);
+      ret = new MicroBreakWindow(head, ignorable, block_mode);
     }
   else if (break_id == BREAK_ID_REST_BREAK)
     {
@@ -752,7 +779,6 @@ GUI::init_sound_player()
 }
 
 
-
 void
 GUI::core_event_notify(CoreEvent event)
 {
@@ -794,7 +820,6 @@ GUI::start_prelude_window(BreakId break_id)
     }
 
   active_prelude_count = num_heads;
-
 }
 
 
@@ -907,19 +932,6 @@ GUI::set_break_progress(int value, int max_value)
 }
 
 
-bool
-GUI::delayed_hide_break_window()
-{
-  bool ret = false;
-  if (active_prelude_count > 0)
-    {
-      core->set_activity_monitor_listener(this);
-      ret = true;
-    }
-  return ret;
-}
-
-
 void
 GUI::set_prelude_stage(PreludeStage stage)
 {
@@ -990,34 +1002,6 @@ GUI::collect_garbage()
 }
 
 
-bool
-GUI::action_notify()
-{
-  TRACE_ENTER("GUI::action_notify");
-  if (dispatcher != NULL)
-    {
-      TRACE_MSG("notify");
-      dispatcher->send_notification();
-    }
-  TRACE_EXIT();
-  return false; // false: kill listener.
-}
-
-
-void
-GUI::on_activity()
-{
-  TRACE_ENTER("GUI::on_activity");
-  if (response != NULL)
-    {
-      TRACE_MSG("notify");
-      response->stop_prelude(active_break_id);
-    }
-  TRACE_EXIT();
-}
-
-
-
 GUI::BlockMode
 GUI::get_block_mode()
 {
