@@ -1,6 +1,6 @@
 // MainWindow.cc --- Main info Window
 //
-// Copyright (C) 2001, 2002, 2003, 2004 Rob Caelers & Raymond Penners
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 // GNU General Public License for more details.
 //
 
-static const char rcsid[] = "$Id: MainWindow.cc,v 1.9 2004/05/13 16:57:21 rcaelers Exp $";
+static const char rcsid[] = "$Id: MainWindow.cc,v 1.36 2006/01/26 18:37:10 rcaelers Exp $";
 
 #include "preinclude.h"
 
@@ -22,6 +22,12 @@ static const char rcsid[] = "$Id: MainWindow.cc,v 1.9 2004/05/13 16:57:21 rcaele
 #include "config.h"
 #endif
 
+#ifdef WIN32
+#include <gdk/gdkwin32.h>
+#include <shellapi.h>
+#include <pbt.h>
+#endif
+
 #include "nls.h"
 #include "debug.hh"
 
@@ -30,6 +36,9 @@ static const char rcsid[] = "$Id: MainWindow.cc,v 1.9 2004/05/13 16:57:21 rcaele
 
 #ifdef HAVE_GNOME
 #include <gnome.h>
+#endif
+
+#if defined(HAVE_GNOME) || defined(HAVE_KDE)
 #include "AppletWindow.hh"
 #endif
 
@@ -54,16 +63,15 @@ static const char rcsid[] = "$Id: MainWindow.cc,v 1.9 2004/05/13 16:57:21 rcaele
 #include "NetworkLogDialog.hh"
 #endif
 
-#ifdef WIN32
-#include <gdk/gdkwin32.h>
-#include <pbt.h>
-#endif
 
 #include "Menus.hh"
 
 #include <gtkmm/menu.h>
 #include <gtkmm/stock.h>
 
+#include "gtk/gtkmenu.h"
+
+
 const string MainWindow::CFG_KEY_MAIN_WINDOW = "gui/main_window";
 const string MainWindow::CFG_KEY_MAIN_WINDOW_ALWAYS_ON_TOP = "gui/main_window/always_on_top";
 
@@ -82,6 +90,8 @@ const string MainWindow::CFG_KEY_MAIN_WINDOW_Y
 const string MainWindow::CFG_KEY_MAIN_WINDOW_HEAD
 = "gui/main_window/head";
 
+
+
 //! Constructor.
 /*!
  *  \param gui the main GUI entry point.
@@ -121,7 +131,6 @@ MainWindow::~MainWindow()
   TRACE_EXIT();
 }
 
-
 //! Initializes the main window.
 void
 MainWindow::init()
@@ -152,7 +161,7 @@ MainWindow::init()
     }
   
   Glib::ListHandle<Glib::RefPtr<Gdk::Pixbuf> > icon_list(icons);
-  set_icon_list(icon_list);
+  Gtk::Window::set_default_icon_list(icon_list);
     
   enabled = TimerBoxControl::is_enabled("main_window");
 #ifdef WIN32
@@ -365,16 +374,15 @@ bool
 MainWindow::on_delete_event(GdkEventAny *)
 {
   TRACE_ENTER("MainWindow::on_delete_event");
-  bool terminate = true;
 
 #ifdef WIN32
   win32_show(false);
-  terminate = false;
 #else
   GUI *gui = GUI::get_instance(); 
   assert(gui != NULL);
   
-#ifdef HAVE_GNOME
+#if defined(HAVE_GNOME) || defined(HAVE_KDE)
+  bool terminate = true;
   AppletWindow *applet = gui->get_applet_window();
   if (applet != NULL)
     {
@@ -393,7 +401,7 @@ MainWindow::on_delete_event(GdkEventAny *)
     }
 #else
   gui->terminate();
-#endif // HAVE_GNOME
+#endif // HAVE_GNOME || HAVE_KDE
 #endif // WIN32
   
   TRACE_EXIT();
@@ -521,6 +529,9 @@ win32_filter_func (void     *xevent,
 {
   TRACE_ENTER("MainWindow::win32_filter_func");
 
+  (void) event;
+  (void) data;
+  
   MSG *msg = (MSG *) xevent;
   GdkFilterReturn ret = GDK_FILTER_CONTINUE;
   switch (msg->message)
@@ -578,6 +589,22 @@ win32_filter_func (void     *xevent,
         menus->on_applet_command((short) msg->wParam);
         ret = GDK_FILTER_REMOVE;
       }
+      break;
+
+    case WM_USER + 1:
+      {
+        GUI *gui = GUI::get_instance(); 
+        assert(gui != NULL);
+
+        MainWindow *main_window = gui->get_main_window();
+        assert(main_window != NULL);
+        
+        TimerBoxControl *win32_timer_box_control = main_window->get_win32_timer_box_control();
+        win32_timer_box_control->force_cycle();
+        
+        ret = GDK_FILTER_REMOVE;
+      }
+      break;
     }
 
   TRACE_EXIT();
@@ -640,6 +667,10 @@ MainWindow::win32_init()
   Menus *menus = Menus::get_instance();
   win32_tray_menu = menus->create_tray_menu();
 
+//   win32_tray_menu->signal_leave_notify_event().connect(MEMBER_SLOT(*this, &MainWindow::win32_on_leave_notify));
+//   win32_tray_menu->signal_enter_notify_event().connect(MEMBER_SLOT(*this, &MainWindow::win32_on_enter_notify));
+//   win32_tray_menu->add_events(Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK);
+
   Gtk::Menu::MenuList &menulist = win32_tray_menu->items();
 
   menulist.push_front(Gtk::Menu_Helpers::StockMenuElem
@@ -660,22 +691,123 @@ MainWindow::win32_exit()
 void
 MainWindow::win32_add_tray_icon()
 {
+  normal_icon = LoadIcon(win32_hinstance, "workrave");
+  quiet_icon = LoadIcon(win32_hinstance, "workravequiet");
+  suspended_icon = LoadIcon(win32_hinstance, "workravesusp");
+
+  memset(&win32_tray_icon, 0, sizeof(NOTIFYICONDATA));
+    
   win32_tray_icon.cbSize = sizeof(NOTIFYICONDATA);
   win32_tray_icon.hWnd = win32_main_hwnd;
   win32_tray_icon.uID = 1;
-  win32_tray_icon.uFlags = NIF_ICON|NIF_TIP|NIF_MESSAGE;
+  win32_tray_icon.uFlags = NIF_ICON|NIF_TIP|NIF_MESSAGE; //|0x00000010; // NIF_INFO;
   win32_tray_icon.uCallbackMessage = MYWM_TRAY_MESSAGE;
-  win32_tray_icon.hIcon = LoadIcon(win32_hinstance, "workrave");
+  win32_tray_icon.hIcon = normal_icon;
   strcpy(win32_tray_icon.szTip, "Workrave");
+
+  // Balloon: not used.
+  //win32_tray_icon.dwInfoFlags = 0; // NIIF_NONE;
+  //win32_tray_icon.uTimeout = 0; // 5 * 1000;
+  //strcpy(win32_tray_icon.szInfoTitle, "Workrave");
+  //strncpy(win32_tray_icon.szInfo, "Workrave", 255);
+
   Shell_NotifyIcon(NIM_ADD, &win32_tray_icon);
   DestroyIcon(win32_tray_icon.hIcon);
 }
 
+void
+MainWindow::win32_set_tray_tooltip(string tip)
+{
+  char *text = NULL;
+  const char *tip_locale = tip.c_str();
+
+  GError *error = NULL;
+  
+  text = g_locale_from_utf8(tip_locale, -1, NULL, NULL, &error);
+
+  if (error != NULL)
+    {
+      TRACE_ENTER("MainWindow::win32_set_tray_tooltip");
+      TRACE_MSG(error->message);
+      TRACE_EXIT();
+      g_error_free(error);
+    } 
+
+  if (text != NULL)
+    {    
+      strncpy(win32_tray_icon.szTip, text, 127);
+    }
+  else
+    {
+      strncpy(win32_tray_icon.szTip, "Workrave", 127);
+    }
+      
+  Shell_NotifyIcon(NIM_MODIFY, &win32_tray_icon);
+}
+
+
+void
+MainWindow::win32_set_tray_icon(TimerBoxView::IconType icon)
+{
+  string file;  
+  switch (icon)
+    {
+    case TimerBoxView::ICON_NORMAL:
+      win32_tray_icon.hIcon = normal_icon;
+      break;
+      
+    case TimerBoxView::ICON_QUIET:
+      win32_tray_icon.hIcon = quiet_icon;
+      break;
+      
+    case TimerBoxView::ICON_SUSPENDED:
+      win32_tray_icon.hIcon = suspended_icon;
+    }
+  
+  Shell_NotifyIcon(NIM_MODIFY, &win32_tray_icon);
+}
+
+/* Taken from Gaim. needs to be gtkmm-ified. */
+/* This is a workaround for a bug in windows GTK+. Clicking outside of the
+   menu does not get rid of it, so instead we get rid of it as soon as the
+   pointer leaves the menu. */
+static gboolean 
+win32_hide_menu(gpointer data)
+{
+	if (data != NULL) {
+		gtk_menu_popdown(GTK_MENU(data));
+	}
+	return FALSE;
+}
+
+static gboolean
+win32_menu_leave_enter(GtkWidget *menu, GdkEventCrossing *event, void *data)
+{
+  TRACE_ENTER("win32_menu_leave_enter");
+  static guint hide_docklet_timer = 0;
+  if (event->type == GDK_LEAVE_NOTIFY && event->detail == GDK_NOTIFY_ANCESTOR) {
+    /* Add some slop so that the menu doesn't annoyingly disappear when mousing around */
+    if (hide_docklet_timer == 0) {
+      hide_docklet_timer = g_timeout_add(500, win32_hide_menu, menu);
+    }
+  } else if (event->type == GDK_ENTER_NOTIFY && event->detail == GDK_NOTIFY_ANCESTOR) {
+    if (hide_docklet_timer != 0) {
+      /* Cancel the hiding if we reenter */
+      
+      g_source_remove(hide_docklet_timer);
+      hide_docklet_timer = 0;
+    }
+  }
+  TRACE_EXIT();
+  return FALSE;
+}
+
 LRESULT CALLBACK
 MainWindow::win32_window_proc(HWND hwnd, UINT uMsg, WPARAM wParam,
                               LPARAM lParam)
 {
   TRACE_ENTER("MainWindow::win32_window_proc");
+  static bool connect_once = false;
   MainWindow *win = (MainWindow *) GetWindowLong(hwnd, GWL_USERDATA);
   if (win != NULL)
     {
@@ -695,11 +827,22 @@ MainWindow::win32_window_proc(HWND hwnd, UINT uMsg, WPARAM wParam,
                 GdkWindow *gdk_window = window->window;
                 HWND phwnd = (HWND) GDK_WINDOW_HWND(gdk_window);
                 SetForegroundWindow(phwnd);
-                win->win32_tray_menu->popup(0, GetTickCount());
+
+                if (!connect_once)
+                  {
+                    // RC: FIXME: remove this c hack HACK 
+                    TRACE_MSG("connect");
+                    g_signal_connect(window, "leave-notify-event", G_CALLBACK(win32_menu_leave_enter), NULL);
+                    g_signal_connect(window, "enter-notify-event", G_CALLBACK(win32_menu_leave_enter), NULL);
+                    connect_once = true;
+                    TRACE_MSG("connect ok");
+                  }
+                
+                win->win32_tray_menu->popup(0,  GetTickCount());
               }
               break;
             case WM_LBUTTONDBLCLK:
-              win->win32_show(true);
+              win->open_window();
               break;
             }
         }
@@ -717,6 +860,44 @@ MainWindow::win32_on_tray_open()
 }
 
 
+bool
+MainWindow::win32_on_leave_notify(GdkEventCrossing *event)
+{
+  TRACE_ENTER("MainWindow:win32_on_leave_notify")
+  if (event->detail == GDK_NOTIFY_ANCESTOR &&
+      !timeout_connection.connected())
+    {
+      timeout_connection = Glib::signal_timeout().connect(MEMBER_SLOT(*this,
+                                                                      &MainWindow::win32_on_hide_timer),
+                                                          500);
+    }
+
+  TRACE_EXIT();
+  return false;
+}
+
+
+bool
+MainWindow::win32_on_enter_notify(GdkEventCrossing *event)
+{
+  TRACE_ENTER("MainWindow:win32_on_enter_notify")
+  if (event->detail == GDK_NOTIFY_ANCESTOR &&
+      timeout_connection.connected())
+    {
+      timeout_connection.disconnect();
+    }
+  TRACE_EXIT();
+  return false;
+}
+
+bool
+MainWindow::win32_on_hide_timer()
+{
+  win32_tray_menu->popdown();
+
+  return false;
+}
+
 #endif
 
 void
