@@ -1,6 +1,6 @@
 // WorkraveApplet.cc
 //
-// Copyright (C) 2002, 2003 Rob Caelers & Raymond Penners
+// Copyright (C) 2002, 2003, 2005 Rob Caelers & Raymond Penners
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 // GNU General Public License for more details.
 //
 
-static const char rcsid[] = "$Id: WorkraveApplet.c,v 1.1 2004/01/28 12:16:44 dotsphinx Exp $";
+static const char rcsid[] = "$Id: WorkraveApplet.c,v 1.3 2005/03/01 17:23:54 rcaelers Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -356,12 +356,21 @@ verb_about(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     "Johannes Rohr <j.rohr@comlink.apc.org>\n"
     "Christian Vejlbo <christian@vejlbo.dk>\n"
     "Mikolaj Machowski <mikmach@wp.pl>\n"
-    "Pablo Rodriguez\n";
+    "Pablo Rodriguez\n"
+    "Rex Tsai <chihchun@linux.org.tw>\n"
+    "Sergey Kirkinsky <ksa@pfr.altai.ru>\n"
+    "Thomas Basset <thomas.basset@netcourrier.com>\n"
+    "Benjamin Siband <translations@wordinfrench.com>\n"
+    "Claudio Ferreira Filho <filhocf@yahoo.com.br>\n"
+    "Morten Lunde <morten.lunde@broadpark.no>\n"
+    "Juraj Kubelka <Juraj.Kubelka@email.cz>\n"
+    "Artūras Šlajus <x11@h2o.sky.lt>\n"
+    "Haggai Eran <he3@bezeqint.net>\n";
 
   GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(WORKRAVE_DATADIR "/images/workrave.png", NULL);  
   gtk_widget_show (gnome_about_new
                    ("Workrave Applet", VERSION,
-                    "Copyright 2001-2003 Rob Caelers & Raymond Penners",
+                    "Copyright 2001-2005 Rob Caelers & Raymond Penners",
                     _("This program assists in the prevention and recovery"
                       " of Repetitive Strain Injury (RSI)."),
                     (const gchar **) authors,
