@@ -1,6 +1,6 @@
 // Exercise.cc --- Exercises
 //
-// Copyright (C) 2002, 2003, 2004 Raymond Penners <raymond@dotsphinx.com>
+// Copyright (C) 2002, 2003, 2004, 2005 Raymond Penners <raymond@dotsphinx.com>
 // All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// $Id: Exercise.cc,v 1.3 2004/07/06 20:37:12 rcaelers Exp $
+// $Id: Exercise.cc,v 1.6 2005/10/26 18:51:24 rcaelers Exp $
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -164,12 +164,18 @@ exercise_parse_update_i18n_attribute(const GList *languages,
         {
           const gchar *lang = (const gchar *) langs->data;
 
+#if 1         
           // FIXME: this is a hack because MinGW's winnt has
           // wrong sublanguage codes...
           if (! strcmp(lang, "pt_PT"))
             {
               lang = "pt_BR";
             }
+          else if (! strcmp(lang, "pt_BR"))
+            {
+              lang = "pt_PT";
+            }
+#endif
           
           if (! strncmp(lang, nl, nl_len))
             {
